@@ -1,20 +1,25 @@
 """Rasterio's GDAL/AWS environment"""
 
-import attr
+from contextlib import ExitStack
 from functools import wraps, total_ordering
+from inspect import getfullargspec as getargspec
 import logging
 import os
 import re
 import threading
 import warnings
 
-from rasterio._env import (
-        GDALEnv, get_gdal_config, set_gdal_config,
-        GDALDataFinder, PROJDataFinder, set_proj_data_search_path)
-from rasterio.compat import string_types, getargspec
-from rasterio.errors import (
-    EnvError, GDALVersionError, RasterioDeprecationWarning)
-from rasterio.session import Session, DummySession
+import attr
+
+import rasterio._loading
+with rasterio._loading.add_gdal_dll_directories():
+    from rasterio._env import (
+            GDALEnv, get_gdal_config, set_gdal_config,
+            GDALDataFinder, PROJDataFinder, set_proj_data_search_path)
+    from rasterio._version import gdal_version
+    from rasterio.errors import (
+        EnvError, GDALVersionError, RasterioDeprecationWarning)
+    from rasterio.session import Session, DummySession
 
 
 class ThreadEnv(threading.local):
@@ -52,7 +57,7 @@ local = ThreadEnv()
 log = logging.getLogger(__name__)
 
 
-class Env(object):
+class Env:
     """Abstraction for GDAL and AWS configuration
 
     The GDAL library is stateful: it has a registry of format drivers,
@@ -66,11 +71,13 @@ class Env(object):
     is exited, drivers are removed from the registry and other
     configurations are removed.
 
-    Example:
+    Example
+    -------
+    .. code-block:: python
 
-        with rasterio.Env(GDAL_CACHEMAX=512) as env:
+        with rasterio.Env(GDAL_CACHEMAX=128000000) as env:
             # All drivers are registered, GDAL's raster block cache
-            # size is set to 512MB.
+            # size is set to 128 MB.
             # Commence processing...
             ...
             # End of processing.
@@ -98,7 +105,6 @@ class Env(object):
         dict
         """
         return {
-            'CHECK_WITH_INVERT_PROJ': True,
             'GTIFF_IMPLICIT_JPEG_OVR': False,
             "RASTERIO_ENV": True
         }
@@ -348,7 +354,7 @@ def delenv():
     local._env = None
 
 
-class NullContextManager(object):
+class NullContextManager:
 
     def __init__(self):
         pass
@@ -419,8 +425,10 @@ def ensure_env_with_credentials(f):
         else:
             env_ctor = Env.from_defaults
 
-        if isinstance(args[0], str):
-            session_cls = Session.cls_from_path(args[0])
+        fp_arg = kwds.get("fp", None) or args[0]
+
+        if isinstance(fp_arg, str):
+            session_cls = Session.cls_from_path(fp_arg)
 
             if local._env and session_cls.hascreds(getenv()):
                 session_cls = DummySession
@@ -438,7 +446,7 @@ def ensure_env_with_credentials(f):
 
 @attr.s(slots=True)
 @total_ordering
-class GDALVersion(object):
+class GDALVersion:
     """Convenience class for obtaining GDAL major and minor version components
     and comparing between versions.  This is highly simplistic and assumes a
     very normal numbering scheme for versions and ignores everything except
@@ -478,7 +486,7 @@ class GDALVersion(object):
             return input
         if isinstance(input, tuple):
             return cls(*input)
-        elif isinstance(input, string_types):
+        elif isinstance(input, str):
             # Extract major and minor version components.
             # alpha, beta, rc suffixes ignored
             match = re.search(r'^\d+\.\d+', input)
@@ -494,7 +502,6 @@ class GDALVersion(object):
     @classmethod
     def runtime(cls):
         """Return GDALVersion of current GDAL runtime"""
-        from rasterio._base import gdal_version  # to avoid circular import
         return cls.parse(gdal_version())
 
     def at_least(self, other):
@@ -508,22 +515,27 @@ def require_gdal_version(version, param=None, values=None, is_max_version=False,
     by the runtime version of GDAL.  Raises GDALVersionError if conditions
     are not met.
 
-    Examples:
-    \b
+    Examples
+    --------
+
+    .. code-block:: python
+
         @require_gdal_version('2.2')
         def some_func():
 
     calling `some_func` with a runtime version of GDAL that is < 2.2 raises a
     GDALVersionErorr.
 
-    \b
+    .. code-block:: python
+
         @require_gdal_version('2.2', param='foo')
         def some_func(foo='bar'):
 
     calling `some_func` with parameter `foo` of any value on GDAL < 2.2 raises
     a GDALVersionError.
 
-    \b
+    .. code-block:: python
+
         @require_gdal_version('2.2', param='foo', values=('bar',))
         def some_func(foo=None):
 
@@ -620,31 +632,36 @@ if 'GDAL_DATA' not in os.environ:
     path = GDALDataFinder().search_wheel()
 
     if path:
-        os.environ['GDAL_DATA'] = path
-        log.debug("GDAL data found in package, GDAL_DATA set to %r.", path)
+        log.debug("GDAL data found in package: path=%r.", path)
+        set_gdal_config("GDAL_DATA", path)
 
-    # See https://github.com/mapbox/rasterio/issues/1631.
+    # See https://github.com/rasterio/rasterio/issues/1631.
     elif GDALDataFinder().find_file("header.dxf"):
-        log.debug("GDAL data files are available at built-in paths")
+        log.debug("GDAL data files are available at built-in paths.")
 
     else:
         path = GDALDataFinder().search()
 
         if path:
-            os.environ['GDAL_DATA'] = path
-            log.debug("GDAL_DATA not found in environment, set to %r.", path)
+            set_gdal_config("GDAL_DATA", path)
+            log.debug("GDAL data found in other locations: path=%r.", path)
 
 if "PROJ_LIB" in os.environ:
     path = os.environ["PROJ_LIB"]
     set_proj_data_search_path(path)
 
-# See https://github.com/mapbox/rasterio/issues/1631.
+elif PROJDataFinder().search_wheel():
+    path = PROJDataFinder().search_wheel()
+    log.debug("PROJ data found in package: path=%r.", path)
+    set_proj_data_search_path(path)
+
+# See https://github.com/rasterio/rasterio/issues/1631.
 elif PROJDataFinder().has_data():
-    log.debug("PROJ data files are available at built-in paths")
+    log.debug("PROJ data files are available at built-in paths.")
 
 else:
     path = PROJDataFinder().search()
 
     if path:
-        log.debug("PROJ data not found in environment, setting to %r.", path)
+        log.debug("PROJ data found in other locations: path=%r.", path)
         set_proj_data_search_path(path)
