@@ -1,6 +1,5 @@
 """Functions for working with features in a raster dataset."""
 
-
 import logging
 import math
 import os
@@ -8,17 +7,20 @@ import warnings
 
 import numpy as np
 
-import rasterio
-from rasterio.dtypes import validate_dtype, can_cast_dtype, get_minimum_dtype
-from rasterio.enums import MergeAlg
-from rasterio.env import ensure_env
-from rasterio.errors import ShapeSkipWarning
-from rasterio._features import _shapes, _sieve, _rasterize, _bounds
-from rasterio import warp
-from rasterio.rio.helpers import coords
-from rasterio.transform import Affine
-from rasterio.transform import IDENTITY, guard_transform, rowcol
-from rasterio.windows import Window
+import rasterio._loading
+
+with rasterio._loading.add_gdal_dll_directories():
+    import rasterio
+    from rasterio.dtypes import validate_dtype, can_cast_dtype, get_minimum_dtype, _getnpdtype
+    from rasterio.enums import MergeAlg
+    from rasterio.env import ensure_env, GDALVersion
+    from rasterio.errors import ShapeSkipWarning
+    from rasterio._features import _shapes, _sieve, _rasterize, _bounds
+    from rasterio import warp
+    from rasterio.rio.helpers import coords
+    from rasterio.transform import Affine
+    from rasterio.transform import IDENTITY, guard_transform, rowcol
+    from rasterio.windows import Window
 
 log = logging.getLogger(__name__)
 
@@ -191,11 +193,11 @@ def rasterize(
 
     Parameters
     ----------
-    shapes : iterable of (`geometry`, `value`) pairs or iterable over
-        geometries. The `geometry` can either be an object that
-        implements the geo interface or GeoJSON-like object. If no
-        `value` is provided the `default_value` will be used. If `value`
-        is `None` the `fill` value will be used.
+    shapes : iterable of (`geometry`, `value`) pairs or geometries
+        The `geometry` can either be an object that implements the geo
+        interface or GeoJSON-like object. If no `value` is provided
+        the `default_value` will be used. If `value` is `None` the
+        `fill` value will be used.
     out_shape : tuple or list with 2 integers
         Shape of output numpy ndarray.
     fill : int or float, optional
@@ -251,6 +253,8 @@ def rasterize(
     valid_dtypes = (
         'int16', 'int32', 'uint8', 'uint16', 'uint32', 'float32', 'float64'
     )
+    if GDALVersion.runtime().at_least("3.5"):
+        valid_dtypes = valid_dtypes + ("int64", "uint64")
 
     def format_invalid_dtype(param):
         return '{0} dtype must be one of: {1}'.format(
@@ -276,7 +280,7 @@ def rasterize(
         if dtype is not None and not can_cast_dtype(default_value_array, dtype):
             raise ValueError(format_cast_error('default_vaue', dtype))
 
-    if dtype is not None and np.dtype(dtype).name not in valid_dtypes:
+    if dtype is not None and _getnpdtype(dtype).name not in valid_dtypes:
         raise ValueError(format_invalid_dtype('dtype'))
 
     valid_shapes = []
@@ -299,7 +303,7 @@ def rasterize(
             if geom_type == 'GeometryCollection':
                 # GeometryCollections need to be handled as individual parts to
                 # avoid holes in output:
-                # https://github.com/mapbox/rasterio/issues/1253.
+                # https://github.com/rasterio/rasterio/issues/1253.
                 # Only 1-level deep since GeoJSON spec discourages nested
                 # GeometryCollections
                 for part in geom['geometries']:
@@ -332,7 +336,7 @@ def rasterize(
         raise ValueError(format_cast_error('shape values', dtype))
 
     if out is not None:
-        if np.dtype(out.dtype).name not in valid_dtypes:
+        if _getnpdtype(out.dtype).name not in valid_dtypes:
             raise ValueError(format_invalid_dtype('out'))
 
         if not can_cast_dtype(shape_values, out.dtype):
@@ -438,48 +442,21 @@ def geometry_window(
 
     """
 
-    if pad_x:
-        pad_x = abs(pad_x * dataset.res[0])
+    all_bounds = [bounds(shape, transform=~dataset.transform) for shape in shapes]
 
-    if pad_y:
-        pad_y = abs(pad_y * dataset.res[1])
-
-    all_bounds = [bounds(shape) for shape in shapes]
-
-    xs = [
+    cols = [
         x
         for (left, bottom, right, top) in all_bounds
         for x in (left - pad_x, right + pad_x, right + pad_x, left - pad_x)
     ]
-    ys = [
+    rows = [
         y
         for (left, bottom, right, top) in all_bounds
-        for y in (top + pad_y, top + pad_y, bottom - pad_x, bottom - pad_x)
+        for y in (top - pad_y, top - pad_y, bottom + pad_y, bottom + pad_y)
     ]
 
-    rows1, cols1 = rowcol(
-        dataset.transform, xs, ys, op=math.floor, precision=pixel_precision
-    )
-
-    if isinstance(rows1, (int, float)):
-        rows1 = [rows1]
-    if isinstance(cols1, (int, float)):
-        cols1 = [cols1]
-
-    rows2, cols2 = rowcol(
-        dataset.transform, xs, ys, op=math.ceil, precision=pixel_precision
-    )
-
-    if isinstance(rows2, (int, float)):
-        rows2 = [rows2]
-    if isinstance(cols2, (int, float)):
-        cols2 = [cols2]
-
-    rows = rows1 + rows2
-    cols = cols1 + cols2
-
-    row_start, row_stop = min(rows), max(rows)
-    col_start, col_stop = min(cols), max(cols)
+    row_start, row_stop = int(math.floor(min(rows))), int(math.ceil(max(rows)))
+    col_start, col_stop = int(math.floor(min(cols))), int(math.ceil(max(cols)))
 
     window = Window(
         col_off=col_start,
